module OlympicSpec
  module Schema
    def self.load
      ActiveRecord::Schema.define do
        self.verbose = false
        create_table :teams do |t|
          t.string :name
          t.float :rating, null: false, default: 1500
          t.float :derivation, null: false, default: 350
        end

        create_table :tournaments do |t|
          t.string :type
          t.integer :root_id
        end

        create_table :teams_tournaments, id: false do |t|
          t.integer :team_id
          t.integer :tournament_id
        end

        create_table :matches_teams, id: false do |t|
          t.integer :match_id
          t.integer :team_id
        end

        create_table :sources do |t|
          t.integer :match_id
          t.integer :source_id
          t.string :source_type
        end

        create_table :matches do |t|
          t.integer :tournament_id
          t.integer :winner_id
          t.integer :depth
          t.integer :offset
          t.integer :value
          t.text :data
        end
      end
    end

    def self.unload
      ActiveRecord::Schema.define do
        self.verbose = false
        drop_table :teams
        drop_table :tournaments
        drop_table :teams_tournaments
        drop_table :matches_teams
        drop_table :sources
        drop_table :matches
      end
    end

    def self.connect
      ActiveRecord::Base.
        establish_connection(adapter: "sqlite3", database: ":memory:")
    end
  end
end
