describe Olympic::Settings do

  subject { described_class.new(team_class: "Hello", foo: "bar") }

  describe "#build" do

    it "yields itself" do
      expect { |b| subject.build(&b) }.to yield_with_args(subject)
    end

    it "returns itself" do
      expect(subject.build {}).to be subject
    end
  end

  describe "#[]" do
    it "accesses using symbols" do
      expect(subject["foo"]).to eq "bar"
      expect(subject[:foo]).to be subject["foo"]
    end

    it "raises on a missing key" do
      expect { subject[:not_here] }.to raise_error KeyError
    end
  end

  describe "#[]=" do
    it "sets with a symbol key" do
      subject["may"] = "not"
      expect(subject[:may]).to eq "not"
    end
  end

  describe "#method_missing" do
    it "accesses the settings" do
      expect(subject.foo).to eq "bar"
    end

    it "sets with a symbol key" do
      subject.baz = "foo"
      expect(subject.baz).to eq "foo"
    end

    it "raises otherwise" do
      expect { subject.foo {} }.to raise_error
      expect { subject.foo(1) }.to raise_error
      expect { subject.foo(1, 2) }.to raise_error
      expect { subject.foo! }.to raise_error
      expect { subject.foo? }.to raise_error
      expect { subject.send(:foo=, 1, 2) }.to raise_error
    end
  end

  describe ".settings" do
    specify { expect(described_class.settings).
      to be_a described_class }

    it "persists" do
      expect(Olympic::Settings.settings).
        to be Olympic::Settings.settings
    end
  end

  describe ".configure" do

    it "yields a Settings instance" do
      expect { |b| described_class.configure(&b) }.
        to yield_with_args(Olympic::Settings)
    end

    it "returns a settings instance" do
      expect(described_class.configure {}).to be_a Olympic::Settings
      expect(described_class.configure {}).
        to be described_class.settings
    end
  end

  describe ".[]" do
    it "accesses the settings instance" do
      expect(described_class[:team_class]).
        to be described_class.settings[:team_class]
    end
  end

end
